<?php 
/**
 * @file
 * Pie chart.
 * Displays a chart containing donation totals broken out by source.
 */

// create the ctools plugin
$plugin = array(
  'single' => TRUE, 
  'title' => t('Donations: Donations by Source Chart'), 
  'description' => t('Displays a pie chart breaking down donations by source.'), 
  'category' => t('CRM Core Donations'),
  'admin info' => 'crm_core_donation_donation_by_source_chart_info',
  'edit form' => 'crm_core_donation_donation_by_source_chart_form',
  'render callback' => 'crm_core_donation_donation_by_source_chart',
  'defaults' => array( 
    'height' => variable_get('crm_core_donation_default_height', '250'),
    'color_series' => variable_get('crm_core_donation_series_colors', ''),
    'display_labels' => variable_get('crm_core_donation_display_labels', 0),
    'include_legend' => '0',
  ),
  'all contexts' => TRUE,
);

/**
 * admin info
 */
function crm_core_donation_donation_by_source_chart_info($subtype, $conf, $contexts) {
  $block = new stdClass;
  $block->title = t('Donations: Donations by Source Chart');
  $block->content = t('Displays a pie chart breaking down donations by source.');
  return $block;
}

/**
 * Settings form
 */
function crm_core_donation_donation_by_source_chart_form($form, &$form_state) {
  $conf = $form_state['conf'];
  
  $form['include_legend'] = array(
    '#type' => 'checkbox', 
    '#title' => t('Display a legend'),
    '#description' => t('Check this box to include a legend in your chart.'),
    '#default_value' => !empty($conf['include_legend']) ? $conf['include_legend'] : '0',
    '#return_value' => 1, 
  );
  
  // display settings
  $form['display_settings'] = array(
    '#type' => 'fieldset', 
    '#title' => t('Display settings'), 
    '#weight' => 5, 
    '#collapsible' => FALSE, 
  );

  // height
  $form['display_settings']['height'] = array(
    '#type' => 'textfield',
    '#title' => t('Height'),
    '#description' => t('Please enter the height of the chart, as an integer.'),
    '#default_value' => !empty($conf['height']) ? $conf['height'] : '',
  );
  
  // display labels
  $form['display_labels'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display labels'),
    '#description' => t('Check this box to display labels for every region on your chart.'),
    '#default_value' => !empty($conf['display_labels']) ? $conf['display_labels'] : '0',
    '#return_value' => 1, 
  );
  
  // color series
  $form['display_settings']['color_series'] = array(
    '#type' => 'textarea',
    '#title' => t('Color series'),
    '#description' => t('Please enter the colors to use for the data series in valid #RRGGBB or rgba format. Please separate each value with a new line. Leave blank to use the default colors.'),
    '#default_value' => !empty($conf['color_series']) ? $conf['color_series'] : '',
  );
  
  return $form;
}

/**
 * submit handler for settings form
 */
function crm_core_donation_donation_by_source_chart_form_validate($form, &$form_state) {
  
  // height
  if (!is_numeric($form_state['values']['height'])) {
    form_set_error('height', t('The height of the chart must be entered as a number (i.e. 200).'));
  } 
  
}

/**
 * submit handler for settings form
 */
function crm_core_donation_donation_by_source_chart_form_submit($form, &$form_state) {
  
  // legend
  $form_state['conf']['include_legend'] = $form_state['values']['include_legend'];
  
  // height
  $form_state['conf']['height'] = $form_state['values']['height'];
  
  // colors
  $form_state['conf']['color_series'] = $form_state['values']['color_series'];
  
  // display labels
  $form_state['conf']['display_labels'] = $form_state['values']['display_labels'];
  
}

/**
 * Render callback
 */
function crm_core_donation_donation_by_source_chart($subtype, $conf, $panel_args, $context = NULL) {
  
  // get a list of sources along with total amounts raised
  $sources = db_query("SELECT SUM(fda.field_cmcd_amount_value) AS amount, fds.field_cmcd_source_value AS source
  	FROM {field_data_field_cmcd_amount} fda JOIN {field_data_field_cmcd_source} fds
  	ON fda.entity_id = fds.entity_id GROUP BY source ORDER BY source;");

  // set up a structure for displaying the data
  $source_data = array();
  foreach ($sources as $source) {
    $source_data[] = new flotData(array(array(0, $source->amount)));
    $source_data[sizeof($source_data) - 1]->label = $source->source;
  }

  // create a new pie chart
  $options = new flotStylePie();
      
  // create a radius, make it a donut chart
  $options->series->pie->radius = 1;
  $options->series->pie->innerRadius = 0.5;

  // add the color series
  if ($conf['color_series'] !== '') {
    $options->colors = explode("\n", $conf['color_series']);
  }  
      
  // add the labels
  if ($conf['display_labels'] !== 0) {
    $options->series->pie->label->show = TRUE;
    $options->series->pie->label->radius = 2/3;
    $options->series->pie->label->threshold = 0.01;
    $options->series->pie->label->backgroundOpacity = 1;
  } 
  else {
    $options->series->pie->label->show = FALSE;
  }

  // display the chart
  $content = theme('flot_graph', array(
    'data' => $source_data,
    'options' => $options,
    'element' => array(
      'style' => 'width: 100%; height: ' . (int)$conf['height'] . 'px;',
    ),
    'legend'  => ($conf['include_legend'] === 1) ? TRUE : FALSE,
  ));

  $block = new stdClass();
  $block->title = t('Donations by Source');
  $block->content = $content;
  
  return $block;  
  
}








